package com.stormtales.notevault.network.sync;

import android.content.Context;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.network.NetworkModule;
import com.stormtales.notevault.network.auth.AuthAPI;
import com.stormtales.notevault.network.sync.models.*;
import com.stormtales.notevault.ui.gallery.GalleryViewModel;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class SongSyncService {
    private final SongSyncAPI songSyncAPI;
    private GalleryViewModel syncStatusViewModel;

    public SongSyncService(Context context) {
        songSyncAPI = NetworkModule.getRetrofitInstance(context).create(SongSyncAPI.class);
    }

    public void syncCreatedSongs(List<Song> songs, SongSyncModule.FinishSongCreateSyncingCallback callback) {
        List<SongCreateRequest> createRequests = new ArrayList<>();
        for(Song song : songs) {
            createRequests.add(new SongCreateRequest(song));
        }
        SongCreateBatchRequest songCreateBatchRequest = new SongCreateBatchRequest(createRequests);
        songSyncAPI.syncCreatedSongs(songCreateBatchRequest).enqueue(new Callback<BatchCreateResponse>() {
            @Override
            public void onResponse(Call<BatchCreateResponse> call, Response<BatchCreateResponse> response) {
                callback.finishSongSyncing(response.body());
            }

            @Override
            public void onFailure(Call<BatchCreateResponse> call, Throwable throwable) {

            }
        });
    }

    public void uploadNoteSheetsOfCreatedSongs(List<NoteSheet> noteSheets, BatchCreateResponse batchCreateResponse, SongSyncModule.FinishSongCreateSyncingCallback callback) {
        final int[] responses = {0};
        for(CreateResponse createResponse : batchCreateResponse.getCreateResponses()) {
            for(NoteSheet noteSheet : noteSheets) {
                if(noteSheet.getSongID() == createResponse.getLocalID()) {
                    File imageFile = new File(noteSheet.getLocalFileName());

                    RequestBody serverID = RequestBody.create(MediaType.parse("text/plain"), createResponse.getServerID());
                    RequestBody fileName = RequestBody.create(MediaType.parse("text/plain"), imageFile.getName());

                    RequestBody requestFile = RequestBody.create(MediaType.parse("image/**"), new File(noteSheet.getLocalFileName()));
                    MultipartBody.Part image = MultipartBody.Part.createFormData("image", noteSheet.getLocalFileName(), requestFile);

                    songSyncAPI.uploadNoteSheet(serverID, fileName, image).enqueue(new Callback<UploadResponse>() {
                        @Override
                        public void onResponse(Call<UploadResponse> call, Response<UploadResponse> response) {
                            responses[0]++;
                            if(responses[0] == batchCreateResponse.getCreateResponses().size()) {
                                callback.finishSongSyncing(batchCreateResponse);
                            }
                        }

                        @Override
                        public void onFailure(Call<UploadResponse> call, Throwable throwable) {

                        }
                    });
                }
            }
        }
    }

    public void syncModifiedSongs(List<Song> songs, SongSyncModule.FinishSongSyncingCallback callback) {
        List<SongModifyRequest> modifyRequests = new ArrayList<>();
        for(Song song : songs) {
            modifyRequests.add(new SongModifyRequest(song));
        }
        SongModifyBatchRequest songModifyBatchRequest = new SongModifyBatchRequest(modifyRequests);
        songSyncAPI.syncModifiedSongs(songModifyBatchRequest).enqueue(new Callback<BatchModifyResponse>() {
            @Override
            public void onResponse(Call<BatchModifyResponse> call, Response<BatchModifyResponse> response) {
                callback.finishSongSyncing(response.body());
            }

            @Override
            public void onFailure(Call<BatchModifyResponse> call, Throwable throwable) {

            }
        });
    }

    public void syncDeletedSong(List<Song> songs, SyncDeletedSongsCallback callback) {
        List<String> deleteRequests = new ArrayList<>();
        List<Integer> deletedNotSyncedSongs = new ArrayList<>();
        for(Song song : songs) {
            if(song.getServerID() == null) {
                deletedNotSyncedSongs.add(song.getLocalID());
            } else {
                deleteRequests.add(song.getServerID());
            }
        }

        SongBatchDeleteRequest songBatchDeleteRequest = new SongBatchDeleteRequest(deleteRequests);
        songSyncAPI.syncDeletedSongs(songBatchDeleteRequest).enqueue(new Callback<BatchModifyResponse>() {
            @Override
            public void onResponse(Call<BatchModifyResponse> call, Response<BatchModifyResponse> response) {
                if(response.isSuccessful() && response.body() != null) {
                    callback.finishSongSyncing(response.body().getModifiedServerObjects(), deletedNotSyncedSongs);
                }
            }

            @Override
            public void onFailure(Call<BatchModifyResponse> call, Throwable throwable) {

            }
        });
    }



    public interface SyncDeletedSongsCallback {
        void finishSongSyncing(List<String> remoteDeletedSongs, List<Integer> localDeletedSongs);
    }
}
